#include "SoS2MIPS.h"

/*
bibliothèques MIPS:
    strtoint: convertit une chaine de caractere en entier
        entrée: $a0: adresse de la chaine de caractere
        retour: $t1: entier
    strlen: calcule la longueur d'une chaine de caractere mise dans $a1
        entrée: $a1: adresse de la chaine de caractere
        retour: $t0: longueur de la chaine de caractere
    strlen2: calcule la longueur d'une chaine de caractere mise dans $a2
        entrée: $a2: adresse de la chaine de caractere
        retour: $t3: longueur de la chaine de caractere
    strcompare: compare deux chaines de caractere
        entrée: $a0: adresse de la premiere chaine de caractere
                $a1: adresse de la deuxieme chaine de caractere
        retour: $t0: 0 si les chaines sont identiques, 1 sinon
    strconcat: concatene deux chaines de caractere
        entrée: $a1: adresse de la premiere chaine de caractere (on commence à a1 pour pouvoir allouer la mémoire)
                $a2: adresse de la deuxieme chaine de caractere
        retour: $v1: adresse de la chaine de caractere concatenee
    intostr: convertit un entier en chaine de caractere
        entrée: $a1: entier                                     (on commence à a1 pour pouvoir allouer la mémoire)
        retour: $v1: adresse de la chaine de caractere
*/


void MIPSstrtoint(FILE *f)
{
    //write a string to int converter in MIPS
    fprintf(f,"\nstrtoint:\n");

    //on se place au dernier caractere
    fprintf(f,"\ninitboucle:\n");
    fprintf(f,"lb $t1,0($a0)\n");//on charge le caractere dans $t1
    fprintf(f,"beq $t1,$zero,initfin\n");//si le caractere est nul on sort de la boucle
    fprintf(f,"addi $a0,$a0,1\n");//on incremente l'adresse de la chaine de caractere
    fprintf(f,"j initboucle\n");//on recommence

    fprintf(f,"\ninitfin:\n");//on a fini de se placer au dernier caractere
    fprintf(f,"addi $a0,$a0,-1\n");//on decremente l'adresse de la chaine de caractere
    fprintf(f,"move $t4,$a0\n");//on sauvegarde l'adresse du dernier caractere dans $t2

    //initialisation des variables
    fprintf(f,"\nli $t0,1\n");//on initialise le compteur de puissance a 1
    fprintf(f,"li $t1,0\n");//on initialise le resultat a 0
    fprintf(f,"li $t2,1\n");//on initialise le signe a 1

    fprintf(f,"\nstrtointboucle:\n");
    fprintf(f,"lb $t3,0($t4)\n");//on charge le caractere dans $t3

    //test du caractere
    fprintf(f,"beq $t3,$zero,strtointfin\n");//si le caractere est nul on sort de la boucle
    fprintf(f,"li $t5,10\n");//on met 10 dans $t5
    fprintf(f,"beq $t3,$t5,strtointfin\n");//si le caractere est un retour a la ligne on sort de la boucle
    fprintf(f,"beq $t3,45,strtointnegatif\n");//si le caractere est - on met le signe a -1
    fprintf(f,"beq $t3,43,strtointpositif\n");//si le caractere est + on met le signe a 1
    fprintf(f,"blt $t3,48,error\n");//si le caractere est inferieur a 0 on affiche une erreur
    fprintf(f,"bgt $t3,57,error\n");//si le caractere est superieur a 9 on affiche une erreur

    //calcul du resultat
    fprintf(f,"addi $t3,$t3,-48\n");//on soustrait 48 au caractere pour avoir la valeur numerique
    fprintf(f,"mult $t3,$t0\n");//on multiplie le caractere par la puissance
    fprintf(f,"mflo $t3\n");//on met le resultat dans $t3
    fprintf(f,"add $t1,$t1,$t3\n");//on ajoute le resultat a la valeur
    fprintf(f,"addi $s0,$0,10\n");//on met 10 dans $s0
    fprintf(f,"mult $t0,$s0\n");//on multiplie la puissance par 10
    fprintf(f,"mflo $t0\n");//on met le resultat dans $t0
    fprintf(f,"addi $t4,$t4,-1\n");//on decremente l'adresse de la chaine de caractere
    fprintf(f,"j strtointboucle\n");//on recommence

    fprintf(f,"\nstrtointnegatif:\n");
    fprintf(f,"addi $t2,$t2,-1\n");//on met le signe a -1
    fprintf(f,"j strtointfin\n");//on va à la fin

    fprintf(f,"\nstrtointpositif:\n");
    fprintf(f,"addi $t2,$t2,1\n");//on met le signe a 1
    fprintf(f,"j strtointfin\n");//on va à la fin

    fprintf(f,"\nerror:\n");
    fprintf(f,"li $t1,0\n");//si erreur on met retourne 0
    fprintf(f,"la $a0,errorstrtoint\n");
    fprintf(f,"li $v0,4\n");
    fprintf(f,"syscall\n");
    fprintf(f,"li $v0,10\n");
    fprintf(f,"syscall          #la fonction échoue, on veut convertir des strings non compatibles\n");

    fprintf(f,"\nstrtointfin:\n");//on a fini de compter
    fprintf(f,"mult $t2,$t1\n");//on multiplie le signe par le compteur de puissance
    fprintf(f,"mflo $t1\n");//on met le resultat dans $t1
    fprintf(f,"li $t0,0\n");//on initialise le compteur a 0
    fprintf(f,"li $t3,0\n");//on initialise le caractere a 0
    fprintf(f,"li $t4,0\n");//on initialise le nombre a 0
    //fprintf(f,"move $v0,$t1\n");//on met le resultat dans $v0
    fprintf(f,"jr $ra       #la fonction a bien terminé\n");//on retourne au programme
}

void MIPSstrlen(FILE *f)
{
    fprintf(f,"\nstrlen:\n");

    fprintf(f,"li $t0,0\n");// compteur de caractere
    fprintf(f,"move $t4,$a1\n");//on sauvegarde l'adresse de la chaine de caractere dans $t4

    fprintf(f,"\nstrlenboucle:\n");
    fprintf(f,"lb $t1,0($a1)\n");//on charge le caractere dans $t1
    fprintf(f,"beq $t1,$zero,strlenfin\n");//si le caractere est nul on sort de la boucle
    fprintf(f,"addi $a1,$a1,1\n");//on incremente l'adresse de la chaine de caractere
    fprintf(f,"addi $t0,$t0,1\n");//on incremente le compteur de caractere
    fprintf(f,"j strlenboucle\n");//on recommence

    fprintf(f,"\nstrlenfin:\n");//on a fini de compter
    fprintf(f,"move $a1,$t4\n");//on remet l'adresse de la chaine de caractere dans $a1
    fprintf(f,"jr $ra\n");//on retourne
}

void MIPSstrlen2(FILE *f)
{
    fprintf(f,"\nstrlen2:\n");

    fprintf(f,"li $t3,0\n");// compteur de caractere
    fprintf(f,"move $t4,$a2\n");//on sauvegarde l'adresse de la chaine de caractere dans $t4

    fprintf(f,"\nstrlen2boucle:\n");
    fprintf(f,"lb $t1,0($a2)\n");//on charge le caractere dans $t1
    fprintf(f,"beq $t1,$zero,strlen2fin\n");//si le caractere est nul on sort de la boucle
    fprintf(f,"addi $a2,$a2,1\n");//on incremente l'adresse de la chaine de caractere
    fprintf(f,"addi $t3,$t3,1\n");//on incremente le compteur de caractere
    fprintf(f,"j strlen2boucle\n");//on recommence

    fprintf(f,"\nstrlen2fin:\n");//on a fini de compter
    fprintf(f,"move $a2,$t4\n");//on remet l'adresse de la chaine de caractere dans $a2
    fprintf(f,"jr $ra\n");//on retourne
}

void MIPSstrcompare(FILE *f)
{
    fprintf(f,"\nstrcompare:\n");

    fprintf(f,"li $t0,0\n");//renvoie 0 si les 2 chaines sont egales
    fprintf(f,"li $t3,10\n");//renvoie 1 si les 2 chaines sont differentes

    fprintf(f,"\nstrcompareboucle:\n");
    fprintf(f,"lb $t1,0($a0)\n");//on charge le 1er caractere dans $t1
    fprintf(f,"lb $t2,0($a1)\n");//on charge le 2eme caractere dans $t2

    //cas particulier du \n
    fprintf(f,"beq $t1,$t3,retour1\n");//si le caractere est \n on sort de la boucle
    fprintf(f,"beq $t2,$t3,retour2\n");//si le caractere est \n on sort de la boucle

    fprintf(f,"bne $t1,$t2,notequal\n");//si les 2 caracteres sont differents on sort de la boucle
    fprintf(f,"beq $t1,$zero,equal\n");//si le caractere est nul on sort de la boucle
    fprintf(f,"beq $t1,$t3,equal\n");//si le caractere est \n on sort de la boucle
    fprintf(f,"addi $a0,$a0,1\n");//on incremente l'adresse de la 1ere chaine de caractere
    fprintf(f,"addi $a1,$a1,1\n");//on incremente l'adresse de la 2eme chaine de caractere
    fprintf(f,"j strcompareboucle\n");//on recommence


    fprintf(f,"\nretour1:\n");
    fprintf(f,"addi $a0,$a0,1\n");//on met 1 dans $t0 pour dire que les 2 chaines sont differentes
    fprintf(f,"j strcompareboucle\n");//on sort de la boucle

    fprintf(f,"\nretour2:\n");
    fprintf(f,"addi $a1,$a1,1\n");//on met 1 dans $t0 pour dire que les 2 chaines sont differentes
    fprintf(f,"j strcompareboucle\n");//on sort de la boucle
    

    fprintf(f,"\nnotequal:\n");
    fprintf(f,"addi $t0,$t0,1\n");//on met 1 dans $t0 pour dire que les 2 chaines sont differentes
    fprintf(f,"j strcomparefin\n");//on sort de la boucle

    fprintf(f,"\nequal:\n");
    fprintf(f,"addi $t0,$t0,0\n");//on met 0 dans $t0 pour dire que les 2 chaines sont egales
    fprintf(f,"j strcomparefin\n");//on sort de la boucle


    fprintf(f,"\nstrcomparefin:\n");//on a fini de comparer
    fprintf(f,"jr $ra\n");//on retourne
}

void MIPSstrconcat(FILE *f)
{
    fprintf(f,"\nstrconcat:\n");

    fprintf(f,"move $t7,$ra\n");        //Store ra car jal

    fprintf(f,"jal strlen\n");//on calcule la taille de la 1ere chaine de caractere dans $t0
    fprintf(f,"jal strlen2\n");//on calcule la taille de la 2eme chaine de caractere dans $t3

    fprintf(f,"move $ra,$t7\n");         //restore ra

    fprintf(f,"add $t0,$t0,$t3\n");//on additionne les 2 tailles
    fprintf(f,"addi $t0,$t0,1\n");//on incremente de 1 pour le caractere nul
    
    fprintf(f,"move $a0,$t0\n");//on met la taille dans $a0
    fprintf(f,"li $v0,9\n");//on met 9 dans $v0 pour allouer de la memoire
    fprintf(f,"syscall\n");//on alloue de la memoire pour la chaine de caractere concatenee
    fprintf(f,"move $v1,$v0\n");//on met l'adresse de la chaine de caractere concatenee dans $v1

    fprintf(f,"\nstrconcatboucle:\n");
    fprintf(f,"lb $t1,0($a1)\n");//on charge le caractere dans $t1
    fprintf(f,"beq $t1,$zero,boucledeuxiemechaine\n");//si le caractere est nul on sort de la boucle
    fprintf(f,"sb $t1,0($v0)\n");//on ecrit le caractere dans la chaine de caractere concatenee
    fprintf(f,"addi $a1,$a1,1\n");//on incremente l'adresse de la 1ere chaine de caractere
    fprintf(f,"addi $v0,$v0,1\n");//on incremente l'adresse de la chaine de caractere concatene
    fprintf(f,"j strconcatboucle\n");//on recommence

    fprintf(f,"\nboucledeuxiemechaine:\n");
    fprintf(f,"lb $t1,0($a2)\n");//on charge le caractere dans $t1
    fprintf(f,"beq $t1,$zero,strconcatfin\n");//si le caractere est nul on sort de la boucle
    fprintf(f,"sb $t1,0($v0)\n");//on ecrit le caractere dans la chaine de caractere concatenee
    fprintf(f,"addi $a2,$a2,1\n");//on incremente l'adresse de la 2eme chaine de caractere
    fprintf(f,"addi $v0,$v0,1\n");//on incremente l'adresse de la chaine de caractere concatene
    fprintf(f,"j boucledeuxiemechaine\n");//on recommence


    fprintf(f,"\nstrconcatfin:\n");
    fprintf(f,"li $t1,0\n");//on met 0 dans $t1 pour dire que le caractere est nul
    fprintf(f,"sb $t1,0($v0)\n");//on ecrit le caractere nul dans la chaine de caractere concatenee
    fprintf(f,"jr $ra\n");//on retourne
}

void MIPSintostr(FILE *f)
{
    fprintf(f,"\nintostr:\n");
    fprintf(f,"li $a0,11\n");//on met la taille dans $a0
    fprintf(f,"li $v0,9\n");//on met 9 dans $v0 pour allouer de la memoire
    fprintf(f,"syscall\n");//on alloue de la memoire pour la chaine de caractere concatenee

    fprintf(f,"bnez $a1,intostrloop     #cas spécifique du zero\n");//si la valeur est nulle on ne va pas dans la boucle
    fprintf(f,"li $t1,48\n");//on met le caractere '0' dans $t1
    fprintf(f,"sb $t1,0($v0)\n");//on ecrit le caractere '0' dans la chaine de caractere concatenee
    fprintf(f,"sb $zero,1($v0)\n");//on ecrit le caractere nul dans la chaine de caractere concatenee
    fprintf(f,"jr $ra\n");//on retourne

    //cas spécifiques
    fprintf(f,"\nintostrloop:\n");
    fprintf(f,"move $t1,$a1\n");//on charge la valeur dans $t1
    fprintf(f,"blt $t1,$zero,intostrnegatif\n");//si la valeur est negative on va dans la boucle negative

    //boucle pour aller à la fin de la chaine de caractere
    fprintf(f,"li $t2,10\n");//on met 10 dans $t1
    fprintf(f,"\nloopendadd:\n");
    fprintf(f,"beq $t2,$zero,finloopendadd\n");//si la valeur est nulle on sort de la boucle
    fprintf(f,"addi $v0,$v0,1\n");//on incremente l'adresse de la chaine de caractere concatene
    fprintf(f,"addi $t2,$t2,-1\n");//on decremente $t2
    fprintf(f,"j loopendadd\n");//on recommence

    fprintf(f,"\nfinloopendadd:\n");
    fprintf(f,"move $v1,$v0\n");//on met l'adresse de la chaine de caractere concatenee dans $v1
    fprintf(f,"addi $v1,$v1,1\n");//on incremente l'adresse de la chaine de caractere concatenee
    fprintf(f,"sb $zero,0($v1)\n");//on met le caractere nul à la fin de la chaine de caractere


    //boucle pour écrire la chaine de caractere
    fprintf(f,"\nloopnonzero:\n");
    fprintf(f,"beq $t1,$zero,intostrfin\n");//si la valeur est nulle on sort de la boucle

    //calcul de l'entier à écrire
    fprintf(f,"li $t2,10\n");//on met 10 dans $t2
    fprintf(f,"div $t1,$t2\n");//on divise la valeur par 10
    fprintf(f,"mfhi $t3\n");//on met le reste dans $t3
    fprintf(f,"mflo $t1\n");//on met le quotient dans $t1
    
    //écriture dans le buffer
    fprintf(f,"addi $t3,$t3,48\n");//on ajoute 48 au reste pour avoir le caractere correspondant
    fprintf(f,"sb $t3,0($v0)\n");//on ecrit le caractere dans la chaine de caractere concatenee
    fprintf(f,"subi $v0,$v0,1\n");//on incremente l'adresse de la chaine de caractere concatene
    fprintf(f,"j loopnonzero\n");//on recommence

    //cas négatif
    fprintf(f,"\nintostrnegatif:        #if the number is <0\n");
    fprintf(f,"li $t2,45\n");//on met le caractere '-' dans $t2
    fprintf(f,"j loopnonzero\n");//on recommence

    //fin négatif
    fprintf(f,"\nnegfin:\n");
    fprintf(f,"sb $t2,0($v0)\n");
    fprintf(f,"jr $ra\n");//on retourne

    //fin
    fprintf(f,"\nintostrfin:\n");
    fprintf(f,"bne $t2,$zero,negfin\n");//on decremente l'adresse de la chaine de caractere concatenee
    fprintf(f,"addi $v0,$v0,1\n");//on incremente l'adresse de la chaine de caractere concatenee
    fprintf(f,"jr $ra\n");//on retourne
}

void MIPSread(FILE *f)
{
    fprintf(f,"\n_read:\n");
    fprintf(f,"li $v0,8\n");
    fprintf(f,"li $a1,31\n");
    fprintf(f,"la $a0, la0\n");
    fprintf(f,"syscall\n"); 


    fprintf(f,"li $t0,0\n");
    fprintf(f,"la $9,la0\n");
    fprintf(f,"li $t3,10\n");       //\n

    fprintf(f,"\n_readloop: 			#taille du buffer lu \n");
    fprintf(f,"lb $t2 , ($9)\n");
    fprintf(f,"beq $t2, $0 , _readexit\n");
    fprintf(f,"beq $t2, $t3 , _readNL\n");
    fprintf(f,"addi $t0, $t0, 1\n");
    fprintf(f,"addi $9 , $t1, 1\n");
    fprintf(f,"j _readloop\n");

    fprintf(f,"_readNL:\n");
    fprintf(f,"sb $0,($9)");
    
    fprintf(f,"\n_readexit : \n");
    fprintf(f,"li $v0, 9\n");
    fprintf(f,"move $a0 , $t0			#alloue\n");
    fprintf(f,"syscall\n");

    fprintf(f,"la $9,la0\n");
    fprintf(f,"move $11, $v0\n");

    fprintf(f,"\n_readloop2 :\n");
    fprintf(f,"beq $t0,$0 , _readexit2\n");
    fprintf(f,"lb $t2, ($9)\n");
    fprintf(f,"sb $t2, ($v0)\n");
    fprintf(f,"addi $9,$9,1\n");
    fprintf(f,"addi $v0,$v0,1\n");
    fprintf(f,"addi $t0,$t0,-1\n");
    fprintf(f,"j _readloop2\n");

    fprintf(f,"\n _readexit2 :\n");

    fprintf(f,"jr $31					#resulat ds $11\n");
    fprintf(f,"#the value allocated is in $11\n");
}

/*
    FIN DE LA BIBLIOTHEQUE MIPS
*/


//reserver la memoire pour les symboles et pour les labels 
void labelprint(struct labels l, int size_symb,FILE *f)
{
    fprintf(f, ".data\n .space %i   #place pour les symboles\n #place pour les lables de chaine de charactere\n", size_symb*4);
    fprintf(f,"errorstrtoint : .asciiz \"Ce n'est pas un nombre desolé\" \n");
    for (int i = 1; i < l.cur_place; i++)
        fprintf(f, "la%i : .asciiz \"%s\"\n", i, l.tab[i]);

    fprintf(f, "la0: .space 32         #the buffer for the read buffer of siez 32\n  \n\n .text\n");
}

/*
    Fonction qui permet de generer le code MIPS
    param:
        quad: la structure qui contient les quadruplets
        tabsymbole: la structure qui contient les symboles
        labels: la structure qui contient les labels
    sortie:
        codeMIPS.s: le fichier contenant le code MIPS
*/

void il2MIPS(struct quad quad, struct tabsymbole tabsymbole, struct labels labels)
{
    FILE *f=fopen("codeMIPS.s","w");
    if(f==NULL)
    {
        printf("Error opening file");
        exit(1);
    }

    //reservation of memory (.data)
    int size_symb = 0;

    for (int i = 0; i < tabsymbole.size; i++)

        if (tabsymbole.tab[i])

            size_symb += tabsymbole.tab[i]->nb;

    labelprint(labels, size_symb,f);


    int i=0;
    for(i=0;i<quad.next;i++)
    {
        fprintf(f,"a%i:\n",i);

        switch(quad.quadrup[i].instruction)
        {
            case GOTO:
                if(quad.quadrup[i].type==1)
                {
                    if(quad.quadrup[i].zero.s==NULL)
                    {
                        fprintf(f,"jal a%i\n",quad.quadrup[i].zero.value);
                    }
                    else
                    {
                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:
                                fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+TEXT_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $s0,$%i\n",quad.quadrup[i].zero.s->isint);
                                break;
                            case 3:
                                fprintf(f,"la $s0,la%i\n",quad.quadrup[i].zero.s->memory_place);
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }

                        //jalr $s0
                        fprintf(f,"jalr $s0\n");
                    }
                }
                else
                {
                    if(quad.quadrup[i].zero.s==NULL)
                    {
                        fprintf(f,"j a%i\n",quad.quadrup[i].zero.value);
                    }
                    else
                    {
                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:

                                fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $s0,$%i\n",quad.quadrup[i].zero.s->isint);
                                break;
                            case 3:
                                fprintf(f,"la $s0,la%i\n",quad.quadrup[i].zero.s->memory_place);
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }

                        //jr $s0
                        fprintf(f,"jr $s0\n");
                    }
                }
                break;
            case AFF:
                if(quad.quadrup[i].zero.s==NULL)
                {
                    printf("Error: variable not found \n");
                    exit(1);
                }
                switch(quad.quadrup[i].type)
                {
                    case -2://affectation indirecte de type lw a,$i
                        fprintf(f,"move $s0,$%i\n",quad.quadrup[i].zero.s->isint);          //zero forcement un registre
                        if(!quad.quadrup[i].one.s)
                                fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].one.value+DATA_SEGMENT);

                        else
                        {
                            switch(quad.quadrup[i].one.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s1,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s1,$%i\n",quad.quadrup[i].one.s->isint);
                                    break;
                                case 3:
                                    fprintf(f,"la $s1,la%i\n",quad.quadrup[i].one.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found \n");
                                    exit(1);
                            }
                        }
                        fprintf(f,"sw $s1,($s0)\n");
                        break;
                    case -1://affectation indirecte de type sw $i,a
                        fprintf(f,"lw,$s0,($%i)\n",quad.quadrup[i].one.s->isint);
                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:
                                fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $%i,$0\n",quad.quadrup[i].zero.s->isint);
                                break;
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }
                        break;
                    case 0://affectation simple
                        if(quad.quadrup[i].one.s==NULL)
                        {
                            fprintf(f,"li $s0,%i\n",quad.quadrup[i].one.value);
                            switch(quad.quadrup[i].zero.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        else
                        {
                            switch(quad.quadrup[i].one.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                                    switch(quad.quadrup[i].zero.s->onstack_reg_label)
                                    {
                                        case 0:
                                            fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                            break;
                                        case 1:
                                            fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                            break;
                                        case 2:
                                            fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                            break;

                                        default:
                                            printf("Error: variable not found");
                                            exit(1);
                                    }
                                    break;
                                case 1:
                                    fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                                    switch(quad.quadrup[i].zero.s->onstack_reg_label)
                                    {
                                        case 0:
                                            fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                            break;
                                        case 1:
                                            fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                            break;
                                        case 2:
                                            fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                            break;

                                        default:
                                            printf("Error: variable not found");
                                            exit(1);
                                    }
                                    break;
                                case 2:
                                    fprintf(f,"move $s0,$%i\n",quad.quadrup[i].one.s->isint);
                                    switch(quad.quadrup[i].zero.s->onstack_reg_label)
                                    {
                                        case 0:
                                            fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                            break;
                                        case 1:
                                            fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                            break;
                                        case 2:
                                            fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                            break;
                                        default:
                                            printf("Error: variable not found");
                                            exit(1);
                                    }
                                    break;
                                case 3:
                                    fprintf(f,"la $s0,la%i\n",quad.quadrup[i].one.s->memory_place);
                                    switch(quad.quadrup[i].zero.s->onstack_reg_label)
                                    {
                                        case 0:
                                            fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                            break;
                                        case 1:
                                            fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                            break;
                                        case 2:
                                            fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                            break;
                                        default:
                                            printf("Error: variable not found");
                                            exit(1);
                                    }
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        break;


                    case 1://addition
                        if(quad.quadrup[i].one.s==NULL)
                        {
                            fprintf(f,"li $s0,%i\n",quad.quadrup[i].one.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].one.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s0,$%i\n",quad.quadrup[i].one.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        if(quad.quadrup[i].two.s==NULL)
                        {
                            fprintf(f,"li $s1,%i\n",quad.quadrup[i].two.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].two.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].two.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s1,%i($sp)\n",quad.quadrup[i].two.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s1,$%i\n",quad.quadrup[i].two.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        fprintf(f,"add $s0,$s0,$s1\n");
                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:
                                fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                break;
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }
                        break;


                    case 2://soustraction
                        if(quad.quadrup[i].one.s==NULL)
                        {
                            fprintf(f,"li $s0,%i\n",quad.quadrup[i].one.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].one.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s0,$%i\n",quad.quadrup[i].one.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        if(quad.quadrup[i].two.s==NULL)
                        {
                            fprintf(f,"li $s1,%i\n",quad.quadrup[i].two.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].two.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].two.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s1,%i($sp)\n",quad.quadrup[i].two.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s1,$%i\n",quad.quadrup[i].two.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }

                        fprintf(f,"sub $s0,$s0,$s1\n");
                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:
                                fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                break;
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }
                        break;
                    case 3://multiplication
                        if(quad.quadrup[i].one.s==NULL)
                        {
                            fprintf(f,"li $s0,%i\n",quad.quadrup[i].one.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].one.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s0,$%i\n",quad.quadrup[i].one.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        if(quad.quadrup[i].two.s==NULL)
                        {
                            fprintf(f,"li $s1,%i\n",quad.quadrup[i].two.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].two.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].two.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s1,%i($sp)\n",quad.quadrup[i].two.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s1,$%i\n",quad.quadrup[i].two.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }

                        fprintf(f,"mult $s0,$s1\n");
                        fprintf(f,"mflo $s0\n");

                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:
                                fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                break;
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }
                        break;
                    case 4://division
                        if(quad.quadrup[i].one.s==NULL)
                        {
                            fprintf(f,"li $s0,%i\n",quad.quadrup[i].one.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].one.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s0,$%i\n",quad.quadrup[i].one.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        if(quad.quadrup[i].two.s==NULL)
                        {
                            fprintf(f,"li $s1,%i\n",quad.quadrup[i].two.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].two.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].two.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s1,%i($sp)\n",quad.quadrup[i].two.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s1,$%i\n",quad.quadrup[i].two.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }

                        fprintf(f,"div $s0,$s1\n");
                        fprintf(f,"mflo $s0\n");
                        
                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:
                                fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                break;
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }
                        break;
                    case 42://modulo
                        if(quad.quadrup[i].one.s==NULL)
                        {
                            fprintf(f,"li $s0,%i\n",quad.quadrup[i].one.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].one.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s0,$%i\n",quad.quadrup[i].one.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }
                        if(quad.quadrup[i].two.s==NULL)
                        {
                            fprintf(f,"li $s1,%i\n",quad.quadrup[i].two.value);
                        }
                        else
                        {
                            switch(quad.quadrup[i].two.s->onstack_reg_label)
                            {
                                case 0:
                                    fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].two.s->memory_place+DATA_SEGMENT);
                                    break;
                                case 1:
                                    fprintf(f,"lw $s1,%i($sp)\n",quad.quadrup[i].two.s->memory_place);
                                    break;
                                case 2:
                                    fprintf(f,"move $s1,$%i\n",quad.quadrup[i].two.s->isint);
                                    break;
                                default:
                                    printf("Error: variable not found");
                                    exit(1);
                            }
                        }

                        fprintf(f,"div $s0,$s1\n");
                        fprintf(f,"mfhi $s0\n");
                        switch(quad.quadrup[i].zero.s->onstack_reg_label)
                        {
                            case 0:
                                fprintf(f,"sw $s0,0x%x\n",quad.quadrup[i].zero.s->memory_place+DATA_SEGMENT);
                                break;
                            case 1:
                                fprintf(f,"sw $s0,%i($sp)\n",quad.quadrup[i].zero.s->memory_place);
                                break;
                            case 2:
                                fprintf(f,"move $%i,$s0\n",quad.quadrup[i].zero.s->isint);
                                break;
                            default:
                                printf("Error: variable not found");
                                exit(1);
                        }
                        break;
                }//fin switch type
                break;
            case IF:
                if(quad.quadrup[i].one.s != NULL )
                {
                    switch(quad.quadrup[i].one.s->onstack_reg_label)
                    {
                        case 0:
                            fprintf(f,"lw $s0,0x%x\n",quad.quadrup[i].one.s->memory_place+DATA_SEGMENT);
                            break;
                        case 1:
                            fprintf(f,"lw $s0,%i($sp)\n",quad.quadrup[i].one.s->memory_place);
                            break;
                        case 2:
                            fprintf(f,"move $s0,$%i\n",quad.quadrup[i].one.s->isint);
                            break;
                        case 3:
                            fprintf(f,"la $s0,la%i\n",quad.quadrup[i].one.s->memory_place);
                            break;
                        default:
                            printf("Error: variable not found");
                            exit(1);
                    }
                }
                else
                {
                    fprintf(f,"li $s0,%i\n",quad.quadrup[i].one.value);
                }
                if(quad.quadrup[i].two.s != NULL )
                {
                    switch(quad.quadrup[i].two.s->onstack_reg_label)
                    {
                        case 0:
                            fprintf(f,"lw $s1,0x%x\n",quad.quadrup[i].two.s->memory_place+DATA_SEGMENT);
                            break;
                        case 1:
                            fprintf(f,"lw $s1,%i($sp)\n",quad.quadrup[i].two.s->memory_place);
                            break;
                        case 2:
                            fprintf(f,"move $s1,$%i\n",quad.quadrup[i].two.s->isint);
                            break;
                        case 3:
                            fprintf(f,"la $s1,la%i\n",quad.quadrup[i].two.s->memory_place);
                            break;
                        default:
                            printf("Error: variable not found");
                            exit(1);
                    }
                }
                else
                {
                    fprintf(f,"li $s1,%i\n",quad.quadrup[i].two.value);
                }
                if(quad.quadrup[i].type>=0)
                {
                    if(quad.quadrup[i].one.s && quad.quadrup[i].one.s->onstack_reg_label!=2 && quad.quadrup[i].one.s->isint==0 )
                    {
                        fprintf(f,"move $a0,$s0\n");
                        fprintf(f,"move $t9,$ra\n");
                        fprintf(f,"jal strtoint\n");
                        fprintf(f,"move $ra,$t9\n");
                        fprintf(f,"move $s0,$t1\n");
                    }
                    if(quad.quadrup[i].two.s && quad.quadrup[i].two.s->onstack_reg_label!=2 && quad.quadrup[i].two.s->isint==0 )
                    {
                        fprintf(f,"move $a0,$s1\n");
                        fprintf(f,"move $t9,$ra\n");
                        fprintf(f,"move $s4,$s0\n");    //save the value of s0 beacause use in strtoint
                        
                        fprintf(f,"jal strtoint\n");
                        fprintf(f,"move $s0,$s4\n");    //restore
                        fprintf(f,"move $ra,$t9\n");
                        fprintf(f,"move $s1,$t1\n");
                    }
                }
                switch(quad.quadrup[i].type)
                {
                    case 0://=
                        fprintf(f,"beq $s0,$s1,a%i\n",quad.quadrup[i].zero.value);
                        break;
                    case 1://!=
                        fprintf(f,"bne $s0,$s1,a%i\n",quad.quadrup[i].zero.value);
                        break;
                    case 2://<
                        fprintf(f,"blt $s0,$s1,a%i\n",quad.quadrup[i].zero.value);
                        break;
                    case 3://>
                        fprintf(f,"bgt $s0,$s1,a%i\n",quad.quadrup[i].zero.value);
                        break;
                    case 4://<=
                        fprintf(f,"ble $s0,$s1,a%i\n",quad.quadrup[i].zero.value);
                        break;
                    case 5://>=
                        fprintf(f,"bge $s0,$s1,a%i\n",quad.quadrup[i].zero.value);
                        break;
                    case -2://=
                        fprintf(f,"move $a0,$s0\n");
                        fprintf(f,"move $a1,$s1\n");
                        fprintf(f,"move $t9,$ra\n");
                        fprintf(f,"jal strcompare\n");
                        fprintf(f,"move $ra,$t9\n");
                        fprintf(f,"beq $t0,$0,a%i\n",quad.quadrup[i].zero.value);
                        break;
                    case -1://!=
                        fprintf(f,"move $a0,$s0\n");
                        fprintf(f,"move $a1,$s1\n");
                        fprintf(f,"move $t9,$ra\n");
                        fprintf(f,"jal strcompare\n");
                        fprintf(f,"move $ra,$t9\n");
                        fprintf(f,"bne $t0,$0,a%i\n",quad.quadrup[i].zero.value);
                        break;
                }//fin switch type
                break;
            case SYS:
                fprintf(f,"li $v0,%i\n",quad.quadrup[i].zero.value);
                fprintf(f,"syscall\n");
                break;
            case CALL:
                fprintf(f,"jal %s\n",(char *)quad.quadrup[i].zero.s);
                break;
        }//fin switch instruction
    }//fin for quad

    //écriture des fonctions de bibliothèque
    MIPSread(f);
    MIPSstrcompare(f);
    MIPSstrconcat(f);
    MIPSstrlen(f);
    MIPSstrlen2(f);
    MIPSstrtoint(f);
    MIPSintostr(f);

}//fin fonction